from django.core.exceptions import ValidationError
from django.db import models


class NavigationItem(models.Model):
    """Individual navigation item"""

    order = models.FloatField(default=0, help_text="Order of appearance in navigation")
    section_type = models.CharField(
        max_length=20,
        choices=[
            ("header", "Header Section"),
            ("footer", "Footer Section"),
            ("aside", "Aside Section"),
        ],
        help_text="Section type this navigation item belongs to",
    )
    header_section = models.ForeignKey(
        "HeaderHero",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name="navigation_items",
    )
    footer_section = models.ForeignKey(
        "FooterBottombar",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name="navigation_items",
    )
    aside_section = models.ForeignKey(
        "Aside",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name="navigation_items",
    )
    text = models.CharField(
        blank=True,
        max_length=255,
        help_text="Display text for the navigation item. Must be empty if login and logout texts are provided.",
    )
    login_text = models.CharField(
        blank=True,
        max_length=255,
        help_text="Display text for the Login Link. Both login and logout texts must be provided if one is present.",
    )
    logout_text = models.CharField(
        blank=True,
        max_length=255,
        help_text="Display text for the Logout Link. Both login and logout texts must be provided if one is present.",
    )
    icon = models.CharField(
        max_length=255,
        blank=True,
        help_text="Bootstrap icon class (e.g., 'bi bi-house')",
    )
    href = models.CharField(
        max_length=255, help_text="URL or anchor link for navigation"
    )
    type = models.CharField(
        max_length=50,
        choices=[
            ("normal", "Normal Link"),
            ("heading", "Heading"),
            ("dropdown", "Dropdown"),
            ("nested", "Nested"),
            ("login/logout", "Login/Logout Button"),
        ],
        help_text="Special type of navigation item",
    )
    parent = models.ForeignKey(
        "self",
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        related_name="children",
        help_text="Parent item for dropdown menus",
    )
    authorized = models.JSONField(
        null=True,
        blank=True,
        help_text="List of roles that can see this item (e.g., ['STAFF', 'USER'])",
    )

    class Meta:
        ordering = ["order"]

    def __str__(self):
        return f"{self.section_type} - {self.text}"

    def clean(self):
        if self.section_type == "header" and not self.header_section:
            raise ValidationError("Header navigation items must have a header section")
        elif self.section_type == "footer" and not self.footer_section:
            raise ValidationError("Footer navigation items must have a footer section")
        elif self.section_type == "aside" and not self.aside_section:
            raise ValidationError("Aside navigation items must have an aside section")
        if (self.login_text and not self.logout_text) or (
            self.logout_text and not self.login_text
        ):
            raise ValidationError(
                "Both login and logout texts must be provided if one is present."
            )
        if self.text and (self.login_text or self.logout_text):
            raise ValidationError(
                "Text field must be empty if login and logout texts are provided."
            )
        if (self.login_text or self.logout_text) and self.text:
            raise ValidationError(
                "Login and logout texts must be empty if text field is provided."
            )

    def save(self, *args, **kwargs):
        # Automatically set section_type based on which section is set
        if self.header_section:
            self.section_type = "header"
        elif self.footer_section:
            self.section_type = "footer"
        elif self.aside_section:
            self.section_type = "aside"
        super().save(*args, **kwargs)
